

/*
 *  Author: Arvin Schnell
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <errno.h>
#ifdef __NetBSD__
#  include <machine/endian.h>
#else
#  include <endian.h>
#endif
#include <X11/Xlib.h>
#include <Xm/Xm.h>
#include <iostream>

using std::cerr;

#include "Sample.h"
#include "utils.h"


Sample sample;


Sample::Sample ()
    : pcm (0)
{
}


bool
Sample::open ()
{
    // check length and resolution

    if (!one_bit_set (length) || length < 128) {
	cerr << "error: illegal sample length (must be a 2^n)\n";
	return false;
    }

    if (resolution != 16 && resolution != 32) {
	cerr << "error: illegal sample resolution (must 16 or 32)\n";
	return false;
    }

    // allocate memory

    buffer = new int32_t[2 * length];

    // open DSP

    switch (source) {

	case SOUNDCARD:

	    pcm = PCM::create (device, PCM::RECORD, resolution == 16 ? PCM::S16 : PCM::S32,
			       2, rate, length);
	    if (!pcm) {
		delete[] buffer;
		return false;
	    }

	    break;

	case DISKFILE:

	    pcm = PCM::create (device, PCM::PLAY, resolution == 16 ? PCM::S16 : PCM::S32,
			       2, rate, length);
	    if (!pcm) {
		delete[] buffer;
		return false;
	    }

	    if ((fd = fopen (filename, "rb")) == 0) {
		cerr << "error: can't open file " << filename << ": "
		     << strerror (errno) << '\n';
		delete pcm;
		delete[] buffer;
		return false;
	    }

	    break;

    }

    pcm->info ();

    frames_per_second = (double) (rate) / length;
    frame_count = -1;		// yes!

    return true;
}


void
Sample::calc_dc ()
{
    int64_t left = 0;
    int64_t right = 0;

    int32_t* tmp1 = buffer;
    for (unsigned int i = 0; i < length; i++) {
	left += *tmp1++;
	right += *tmp1++;
    }

    left /= (signed) length;
    right /= (signed) length;

    dc[0] = left;
    dc[1] = right;
}


void
Sample::expand ()
{
    int16_t* tmp1 = (int16_t*)(buffer) + 2 * length;
    int32_t* tmp2 = buffer + 2 * length;
    for (unsigned int i = 0; i < length; i++) {
	*--tmp2 = (*--tmp1) << 16;
	*--tmp2 = (*--tmp1) << 16;
    }
}


bool
Sample::shot ()
{
    switch (source) {

	case SOUNDCARD:

	    pcm->read (buffer, length);

	    if (resolution == 16)
		expand ();

	    break;

	case DISKFILE:

	    size_t j = fread (buffer, 4, length, fd);

	    if (j == 0 || j != length)	// EOF
		return false;

#if __BYTE_ORDER == __LITTLE_ENDIAN
	    pcm->swap_buffer (buffer, length, PCM::S16, 2);
#endif

	    if (resolution == 16) {
		pcm->write (buffer, length);
		expand ();
	    } else {
		expand ();
		pcm->write (buffer, length);
	    }

	    break;
    }

    calc_dc ();

    frame_count++;

    return true;
}


bool
Sample::close ()
{
    delete pcm;
    pcm = 0;

    switch (source) {

	case SOUNDCARD:
	    break;

	case DISKFILE:
	    fclose (fd);
	    break;

    }

    delete[] buffer;

    return true;
}
